;******************************************************************************
;   How to use this file:                                                     *
;   ======================                                                    *
;                                                                             *
;   This file is a basic template for creating Assembly code for a PIC24F     *
;   device.  Copy this file into your project directory and modify or         *
;   add to it as needed.                                                      *
;                                                                             *
;   Add the device linker script (e.g., p24fj128ga010.gld) and the device     *
;   include file (e.g. p24fj128ga010.inc). If you are using this file for     *
;   an application that uses a device other than PIC24fj128ga010 which is     *
;   identified in the list of supported devices, add the linker script and    *
;   include file specific to that device into your project.                   *
;                                                                             *
;   If you plan to use initialized data sections in the application and       *
;   would like to use the start-up module in the run-time library to          *
;   initialize the data sections then use the template file in this folder    *
;   named, "tmp24f_srt.s". Refer to the document "ASM30 Assembler,            *
;   Linker and Utilities User's Guide" for further information on the start-up*
;   module in the run-time library.                                           *
;                                                                             *
;   If interrupts are not used, all code presented for that interrupt         *
;   can be removed or commented out using comment symbols (semi-colons).      *
;                                                                             *
;   For additional information about dsPIC 30F  architecture and language     *
;   tools, refer to the following documents:                                  *
;                                                                             *
;   MPLAB C30 Compiler User's Guide                       : DS51284           *
;   ASM30  Assembler, Linker and Utilities User's Guide	  : DS51317           *
;   PIC24F  Family Reference Manual Sections (mulitple DS numbers)		      *
;   dsPIC 30F Programmer's Reference Manual               : DS70030           *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Author              :                                                    *
;    Company             :                                                    *
;    Filename            :  Labx.s                                            *
;    Date                :  08/25/2006                                        *
;    File Version        :  1.30                                              *
;                                                                             *
;    Other Files Required: p24fj128ga010.gld, p24fj128ga010.inc               *
;    Tools Used:MPLAB GL : 7.40                                               *
;               Compiler : 2.00                                               *
;               Assembler: 1.30                                               *
;               Linker   : 1.30                                               *
;                                                                             *
;    Devices Supported by this file:                                          *
;                        PIC24FJ128GA010                                      *
;                        PIC24FJ128GA008                                      *
;                        PIC24FJ128GA006                                      *
;                        PIC24FJ64GA010                                       *
;                        PIC24FJ64GA008                                       *
;                        PIC24FJ64GA006                                       *
;                        PIC24FJ96GA010                                       *
;                        PIC24FJ96GA008                                       *
;                        PIC24FJ96GA006                                       *
;										                                      *
;                                                                             *
;******************************************************************************
;    Notes:                                                                   *
;    ======                                                                   *
;                                                                             *
;1. The device include file defines bit names and their location within SFR   *
;   words. Replace the path to the include file, if necessary.                *
;   For e.g., to use the p24fj96ga008 device modify the .include directive to *
;   .include "Yourpath\p24fj96ga008.inc"                                      *
;                                                                             *
;2. The "config" macro is present in the device specific '.inc' file and can  *
;   be used to set configuration bits for the 24F device.                     *
;   The '.inc' files also contain examples on using the "config" macro.       *
;   Some examples have been provided in this file. The user application may   *
;   require a different setting for the configuration bits.                   *
;   The PIC24F Family Refernce Manual explains the     						  *
;   function of the configuration bits.                                       *
;                                                                             *
;3. The "__reset" label is the first label in the code section and must be    *
;   declared global. The Stack Pointer (W15) and the Stack Pointer Limit      *
;   register must be initialized to values that are past the run-time heap    *
;   space, so as to avoid corrupting user data. Initializing these registers  *
;   at the "__reset" label ensures that user data is not corrupted            *
;   accidentally. The __SP_init and __SPLIM_init literals are defined by the  *
;   linker. The linker automatically addresses above the run-time heap to     *
;   these symbols. Users can change the __SP_init and __SPLIM_init literals to*
;   suit their application.                                                   *
;                                                                             *
;4. A ".section <section_name>, <attributes>" directive declares a new section*
;   named "<section_name>" with an attribute list that specifies if the       *
;   section is located in Program Space(code), Uninitialized(bss) or          *
;   Initialized Data Space(data). Refer to the document "ASM30 Assembler,     *
;   Linker and Utilities User's Guide" for further details.                   *
;                                                                             *
;5. Initialize W registers: Using uninitialized W registers as Effective      *
;   Addresses(pointers) will cause an "Uninitialized W Register Trap" !       *
;                                                                             *
;6. The label "__T1Interrupt" is defined in the device linker script. The     *
;   label defines the starting location in program space of the Timer1        *
;   interrupt service routine (ISR).                                          *
;   Similarly, the linker script defines labels for all ISRs. Notice, that    *
;   the ISR label names are preceded by two underscore characters.            *
;   When users needs to write ISR code, they must use these labels after      *
;   providing them global scope. The linker will then place the ISR address in*
;   the respective interrupt vector table location in program space.          *
;   Context Save/Restore in ISRs can be performed using the double-word PUSH  *
;   instruction,PUSH.D. User can also make use of MOV.D, PUSH and PUSH.S      *
;   instructions. Refer to PIC24F Family Refernce Manual					  *
;   for further details.                                                      *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Additional Notes:                                                        *
;                                                                             *
;                                                                             *
;                                                                             *
;                                                                             *
;******************************************************************************

        .equ __24fj128ga010, 1
        .include "p24fj128ga010.inc"

;..............................................................................
;Configuration bits:
;..............................................................................

 ;       config __FOSC, CSW_FSCM_OFF & XT_PLL16    ;Turn off clock switching and
                                            ;fail-safe clock monitoring and
                                            ;use the External Clock as the
                                            ;system clock

 ;       config __FWDT, WDT_OFF              ;Turn off Watchdog Timer

 ;       config __FBORPOR, PBOR_ON & BORV_27 & PWRT_16 & MCLR_EN
                                            ;Set Brown-out Reset voltage and
                                            ;and set Power-up Timer to 16msecs
                                            
  ;      config __FGS, CODE_PROT_OFF         ;Set Code Protection Off for the 
                                            ;General Segment

;..............................................................................
;Program Specific Constants (literals used in code)
;..............................................................................

        .equ SAMPLES, 64         ;Number of samples
		.equ S3, 6				;S3 connected to PORTD bit 6
		.equ S4, 13				;S4 connected to PORTD bit 13
		.equ S3Pressed, 0		;Flag bit for S3 Pressed function
		.equ S4Pressed, 1		;Flag bit for S4 Pressed function

	.equiv	FCY, 4000000			;Instruction Cycle Frequency
	.equiv	BAUDRATE, 9600			;Operating Baud Rate
	.equiv	HALFSECND, 2				; Half sec divide value
	.equiv	TMR1PS, 64				; Pre-Scaler for TMR1

;..............................................................................
;Global Declarations:
;..............................................................................

        .global _wreg_init       ;Provide global scope to _wreg_init routine
                                 ;In order to call this routine from a C file,
                                 ;place "wreg_init" in an "extern" declaration
                                 ;in the C file.

        .global __reset          ;The label for the first line of code. 

        .global __T1Interrupt    ;Declare Timer 1 ISR name global



;..............................................................................
;Constants stored in Program space
;..............................................................................

        .section .myconstbuffer, code
        .palign 2                ;Align next word stored in Program space to an
                                 ;address that is a multiple of 2
ps_coeff:
        .hword   0x0002, 0x0003, 0x0005, 0x000A


;..............................................................................
;Constants stored in Program space for PSV access
;..............................................................................
       .section .const, psv
Hello:
		.ascii "Hello world!\n\r\0"



;..............................................................................
;Uninitialized variables in Near data memory (Lower 8Kb of RAM)
;..............................................................................

          .section .nbss, bss, near
var1:     .space 2               ;Example of allocating 1 word of space for
                                 ;variable "var1".

Flags:	.space 2

;..............................................................................
;Code Section in Program Memory
;..............................................................................

.text                             ;Start of Code section
__reset:
        MOV #__SP_init, W15       ;Initalize the Stack Pointer
        MOV #__SPLIM_init, W0     ;Initialize the Stack Pointer Limit Register
        MOV W0, SPLIM
        NOP                       ;Add NOP to follow SPLIM initialization
        
        RCALL _wreg_init           ;Call _wreg_init subroutine
                                  ;Optionally use RCALL instead of CALL
		rcall	InitTMR1			; Initialize TMR1
		rcall	InitPortA			;Initialize PORTA
		rcall	InitUART2			;Initialize UART2
		rcall	InitPSVOperation	;Initialize PSV operation
		bclr	Flags,#S3Pressed	;clr S3 pressed flag				
done:
		btsc	PORTD,#S3			; S3 is pressed?
		bra		done				; no then check again
		rcall	SendPSVString		;yes then send PSV String
Again:
		btsc	Flags,#S3Pressed 	;Flag reset by TMR1 Interrupt?
		goto	Again				; no then check again
		bra		done				; yes then repeat process
;..............................................................................
;Subroutine: Initialization of W registers to 0x0000
;..............................................................................

_wreg_init:
        CLR W0
        MOV W0, W14
        REPEAT #12
        MOV W0, [++W14]
        CLR W14
        RETURN

;..............................................................................
;Subroutine: Delay delays for about half a second @ 4Mips; uses w2
;..............................................................................

Delay:
		clr	w2					;use w2 as a 16-bit counter
LoopDelay:
		dec	w2,w2				; do 65536 times
		bra	z,ExitDelay			; when done exit the loop
		Repeat	#24				; repeat a nop 25 times		
		nop
		bra	LoopDelay			; complete loop
ExitDelay:
		return
;
;..............................................................................
;Subroutine: InitTMR1 initializes TMR1 to interrupt every Quarter second @ 4Mips
;..............................................................................
InitTMR1:
		clr		TMR1
		clr		T1CON
		bset	T1CON,#5		;set pre-scaler divide by 64
		mov		#(FCY/TMR1PS/HALFSECND),w1	; set PR1 for half sec delay
		mov		w1,PR1			
		bset	IEC0,#T1IE		;enable interrupt
		bclr	IFS0,#T1IF		;clr flag
		bset	T1CON,#TON		;start timer 1
		return

;..............................................................................
;Subroutine: InitUART2 initializes UART1 for a 9600 baud tranmission @ Fcyl = 4 Mhz
;..............................................................................
InitUART2:
	clr U2MODE
	clr U2STA
	bset U2MODE, #UARTEN                   	;Enable UART (implies reception)
	mov #(((FCY/BAUDRATE) / 16) - 1), w0       
	mov w0, U2BRG                         	;Initialize BRG 
	bset U2STA, #UTXEN         ;Enable Transmission 
	return 

;..............................................................................
;Subroutine: InitPORTA, initializes PORTA as all outputs and sets LATA.0
;..............................................................................
InitPortA:
		clr		LATA				;Clr Latch A
		clr		TRISA				;Make PORTA outputs
		bset	LATA,#0				;light up LATA.0
		return		

;..............................................................................
;Subroutine: InitPSVOperation, Initializes the PSV operation in this program. 
;We point the the String "Hello World" and transmit it over the UART2 using
; the PSV operation. Uses w0 and w1 registers.
;..............................................................................
InitPSVOperation:

;	<<< Write code below to initialize the PSV operation >>>
		;enable PSV operation
		;get psv page address
		;load in SFR
		;get PSV offset address in w0
		clr		w1	
		return

;..............................................................................
;Subroutine: SendPSVSting Loads the PSV string to the UART2 buffer. 
; Uses w1 and w0 registers
;..............................................................................
SendPSVString:
		bset	Flags,#S3Pressed	; set S3 pressed flag
NextByte:
		mov.b	[w0++],w1			;get byte from PSV string location
		cp0.b	w1					;end of PSV string?
		bra	z,	ResetPtr			; yes then reset the string pointer
TryAgain:
		btsc	U2STA,#UTXBF		; else is tx buffer empty?
		bra		TryAgain			; no then try again	
		mov 	w1,U2TXREG			; load string char in buffer
		mov		w1,LATA				; and display on LEDs
		bra		NextByte			
ResetPtr:
		mov		#psvoffset(Hello),w0 ; reload string pointer
		return

	
;..............................................................................
;Timer 1 Interrupt Service Routine
;Example context save/restore in the ISR performed using PUSH.D/POP.D
;instruction. The instruction pushes two words W4 and W5 on to the stack on
;entry into ISR and pops the two words back into W4 and W5 on exit from the ISR
;..............................................................................

__T1Interrupt:
        PUSH.D W4                  ;Save context using double-word PUSH

        ;<<insert more user code here>>

        BCLR 	IFS0, #T1IF           ;Clear the Timer1 Interrupt flag Status
                                   ;bit.
		nop
		nop

		btss	Flags,#S3Pressed
		retfie
		bclr	Flags,#S3Pressed
        POP.D W4                   ;Retrieve context POP-ping from Stack
        RETFIE                     ;Return from Interrupt Service routine



;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file



